(************************************************************************************
Filename    :   Win32_CameraCone.h
Content     :   Shared functionality for the tracker's cone
Created     :   October 20th, 2014
Author      :   Tom Heath
Copyright   :   Copyright 2014 Oculus, Inc. All Rights reserved.
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*************************************************************************************)
unit UCameraCone;

interface

uses
  Winapi.Windows, System.Math, System.Math.Vectors, OVR_CAPI, OVR_CAPI_Util,
  UDirectXAppUtil, UBasicVR, UCommon;

type
  TCameraCone = class
  private
    v: array[0..7] of TPoint3D;    // Corners of the cone
    SolidModel: TModel;
    WireModel: TModel;
  public
    constructor Create(pbasicVR: TBasicVR);
    destructor Destroy; override;
    function DistToPlane(const [Ref] p, p0, p1, p2: TPoint3D): Single;
    function DistToBoundary(centreEyePosePos: TPoint3D; cameraPose: TOvrPosef): Single;
    procedure RenderToEyeBuffer(vrLayer: TVRLayer; eye: Integer;
      const [Ref] pTrackingState: TOvrTrackingState; const [Ref] pTrackerPose: TOvrTrackerPose;
      proportionVisible: Single);
  end;

implementation

{ TCameraCone }

constructor TCameraCone.Create(pbasicVR: TBasicVR);
var
  hFOV, vFOV, nearZ, farZ: Single;
  baseVec3: TPoint3D;
  i: NativeInt;
  depth: Single;
  ts1, ts2: TTriangleSet;
  color: DWORD;
  boxRadius: Single;
  tex1: TTexture;
  mat1: TMaterial;
  gridDensity: Single;

  procedure AddEdge(i0, i1: Integer);
  begin
    ts1.AddQuad(
      TVertex.Create(v[i0],color,0,0),
      TVertex.Create(v[i1],color,0,0),
      TVertex.Create(v[i1],color,0,0),
      TVertex.Create(v[i1],color,0,0));
  end;

  procedure AddPlane(i0,i1,i2,i3,U0,V0: Integer);
  begin
    ts2.AddQuad(
      TVertex.Create(v[i0],color,gridDensity*v[i0].V[U0],gridDensity*v[i0].V[V0]),
      TVertex.Create(v[i1],color,gridDensity*v[i1].V[U0],gridDensity*v[i1].V[V0]),
      TVertex.Create(v[i2],color,gridDensity*v[i2].V[U0],gridDensity*v[i2].V[V0]),
      TVertex.Create(v[i3],color,gridDensity*v[i3].V[U0],gridDensity*v[i3].V[V0]));
  end;

begin
  inherited Create;

  // Handle the simple case of a single tracker. 
  // Find the corners of the camera cone, from SDK description    // v4-------v5
  hFOV  := pbasicVR.TrackerDescArray[0].FrustumHFovInRadians;     // | \     / |
  vFOV  := pbasicVR.TrackerDescArray[0].FrustumVFovInRadians;     // |  v0-v1  |
  nearZ := pbasicVR.TrackerDescArray[0].FrustumNearZInMeters;     // |  | C |  |
  farZ  := pbasicVR.TrackerDescArray[0].FrustumFarZInMeters;      // |  v2-v3  |
                                                                  // | /     \ |
                                                                  // v6-------v7
  baseVec3 := Point3D(Tan(0.5 * hFOV), Tan(0.5 * vFOV), 1.0);
  v[0] := Point3D( baseVec3.x, -baseVec3.y, 1.0);
  v[4] := v[0];
  v[1] := Point3D(-baseVec3.x, -baseVec3.y, 1.0);
  v[5] := v[1];
  v[2] := Point3D( baseVec3.x,  baseVec3.y, 1.0);
  v[6] := v[2];
  v[3] := Point3D(-baseVec3.x,  baseVec3.y, 1.0);
  v[7] := v[3];

  // Project to near and far planes
  for i := 0 to 7 do
  begin
    depth := IfThen(i < 4, nearZ, farZ);
    v[i].X := v[i].X * depth;
    v[i].Y := v[i].Y * depth;
    v[i].Z := v[i].Z * depth;
  end;

  // Model of wireframe camera and edges
  ts1 := Default(TTriangleSet);
  color := $FFFFFFFF;
  boxRadius := 0.02;
  ts1.AddSolidColorBox(-boxRadius,-boxRadius,-boxRadius,boxRadius,boxRadius,boxRadius,color);
  AddEdge(0,1); AddEdge(1,3); AddEdge(3,2); AddEdge(2,0);
  AddEdge(4,5); AddEdge(5,7); AddEdge(7,6); AddEdge(6,4);
  AddEdge(4,0); AddEdge(5,1); AddEdge(7,3); AddEdge(6,2);
  tex1 := TTexture.Create(False,256,256,TTexture.AUTO_WHITE);
  mat1 := TMaterial.Create(tex1, TMaterial.MAT_WRAP or TMaterial.MAT_WIRE or TMaterial.MAT_ZALWAYS or TMaterial.MAT_NOCULL or TMaterial.MAT_TRANS);
  WireModel := TModel.Create(ts1, Point3D(0, 0, 0), TQuaternion3D.Identity, mat1);

  // Model of solid planes
  ts2 := Default(TTriangleSet);
  gridDensity := 6.0;
  AddPlane(4,0,6,2,2,1); // Left
  AddPlane(1,5,3,7,2,1); // Right
  AddPlane(4,5,0,1,0,2); // Top
  AddPlane(2,3,6,7,0,2); // Bot
  AddPlane(5,4,7,6,0,1); // Back
  SolidModel := TModel.Create(ts2, Point3D(0, 0, 0), TQuaternion3D.Identity,
                  TMaterial.Create(
                    TTexture.Create(False,256,256,TTexture.AUTO_GRID)
                  )
                );
end;

destructor TCameraCone.Destroy;
begin
  SolidModel.Free;
  WireModel.Free;

  inherited;
end;

//-----------------------------------------------------------
function TCameraCone.DistToPlane(const [Ref] p, p0, p1, p2: TPoint3D): Single;
var
  q0, q1, c, q: TPoint3D;
begin
  q0 := p1 - p0;
  q1 := p2 - p0;
  c := q0.CrossProduct(q1).Normalize;
  q := p - p0;
  Result := c.DotProduct(q);
end;

//-----------------------------------------------------------
function TCameraCone.DistToBoundary(centreEyePosePos: TPoint3D; cameraPose: TOvrPosef): Single;
var
  dist: Single;
begin
  // Translate test point back
  centreEyePosePos := centreEyePosePos - TPoint3D(cameraPose.Position);
  // Rotate test point back
  centreEyePosePos := centreEyePosePos * TMatrix3D(QuaternionInverse(TQuaternion3D(cameraPose.Orientation)));

  dist := DistToPlane(centreEyePosePos, v[0], v[3], v[1]); // Front
  dist := Min(dist, DistToPlane(centreEyePosePos, v[5], v[6], v[4]));// Back
  dist := Min(dist, DistToPlane(centreEyePosePos, v[4], v[2], v[0]));// Left
  dist := Min(dist, DistToPlane(centreEyePosePos, v[1], v[7], v[5]));// Right
  dist := Min(dist, DistToPlane(centreEyePosePos, v[4], v[1], v[5]));// Top
  dist := Min(dist, DistToPlane(centreEyePosePos, v[2], v[7], v[3]));// Bottom
  Result := dist;
end;

//-----------------------------------------------------------
procedure TCameraCone.RenderToEyeBuffer(vrLayer: TVRLayer; eye: Integer;
  const [Ref] pTrackingState: TOvrTrackingState; const [Ref] pTrackerPose: TOvrTrackerPose;
  proportionVisible: Single);
var
  tracked: Boolean;
  finalCam: TCamera;
  view, proj, prod: TMatrix3D;
  p: TOvrMatrix4f;
begin
  // Update pose of the models
  // Sure we do this twice, but keeps the code cleaner
  WireModel.Rot := TQuaternion3D(pTrackerPose.Pose.Orientation);
  SolidModel.Rot := WireModel.Rot;
  WireModel.Pos := TPoint3D(pTrackerPose.Pose.Position);
  SolidModel.Pos := WireModel.Pos;
  tracked := pTrackingState.StatusFlags and Ord(ovrStatus_PositionTracked) <> 0;

  // Now render camera volume, using its own static 'zero' camera, so purely rift components
  finalCam := TCamera.Create(TPoint3D(vrLayer.EyeRenderPose[eye].Position),
                             TQuaternion3D(vrLayer.EyeRenderPose[eye].Orientation));
  view := finalCam.GetViewMatrix;
  finalCam.Free;
  p := ovrMatrix4f_Projection(vrLayer.EyeRenderDesc[eye].Fov, 0.01(*0.2*), 1000.0, Ord(ovrProjection_None));
  proj := TMatrix3D(p).Transpose;
  prod := view * proj;

  // Render two components of camera matrix, wireframe last to sort on top
  // and color red if not tracked
  SolidModel.Render(prod, 1, IfThen(tracked, 1.0, 0.0), IfThen(tracked, 1.0, 0.0), proportionVisible, True);
  WireModel.Render(prod, 1, IfThen(tracked, 1.0, 0.0), IfThen(tracked, 1.0, 0.0), proportionVisible, True);
end;

end.